/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2008. All
 * Rights Reserved.
 * 
 * U.S. Government Users Restricted Rights: Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
//
//
package com.ibm.rational.test.lm.spi.virutualization.mock;

import java.net.URI;

import com.ibm.rational.test.lm.spi.ITracer;
import com.ibm.rational.test.lm.spi.IVirtualizationSPI;
import com.ibm.rational.test.lm.spi.common.IRestAPIResponseCodes;
import com.ibm.rational.test.lm.spi.common.IRestResponse;
import com.ibm.rational.test.lm.spi.common.SPIAuthenticationException;
import com.ibm.rational.test.lm.spi.common.SPIClientConnectionException;
import com.ibm.rational.test.lm.spi.common.SPIResourceNotFoundException;
import com.ibm.rational.test.lm.spi.common.SPIURI;
import com.ibm.rational.test.lm.spi.common.URIResourceType;

public class VirtualizationSPI implements IVirtualizationSPI {
	private VirtualizationService _virtualizationService = VirtualizationService.INSTANCE;

	public VirtualizationSPI() {
		// TODO Auto-generated constructor stub
	}

	public IRestResponse delete(URI uri) {
		// Get the meta data from the URI
		SPIURI parser = SPIURI.vmParse(uri);
		String providerHostName = parser.getProviderHostname();
		int providerPort = parser.getProviderPort();
		URIResourceType resourceType = parser.getResourceTypes()[0];
		String resourceId = parser.getResourceID();
		String xml = null;
		RestResponse response = new RestResponse();

		// By default, set the Response code to SC_OK.
		response.setResponseCode(IRestAPIResponseCodes.SC_OK);

		switch (resourceType) {
		case virtualmachine:
			if (resourceId != null && resourceId.length() > 0) {
				boolean success = _virtualizationService.deleteVirtualMachine(
						resourceId);
				// TODO Handle exception
				if (success == false) {
					response
							.setResponseCode(IRestAPIResponseCodes.SC_INTERNAL_SERVER_ERROR);
				}
			}
			break;
		case vmcollection:
			if (resourceId != null && resourceId.length() > 0) {
				boolean success = _virtualizationService
						.deleteVirtualMachineCollection(resourceId);
				// TODO Handle exception
				if (success == false) {
					response
							.setResponseCode(IRestAPIResponseCodes.SC_INTERNAL_SERVER_ERROR);
				}
			}
			break;
		default:
			response.setResponseCode(IRestAPIResponseCodes.SC_NOT_IMPLEMENTED);
		}
		response.setXMLResponse(xml);
		return response;
	}

	public IRestResponse get(URI uri) {
		// Get the meta data from the URI
		SPIURI parser = SPIURI.vmParse(uri);
		String providerHostName = parser.getProviderHostname();
		int providerPort = parser.getProviderPort();
		URIResourceType resourceType = parser.getResourceTypes()[0];
		String resourceId = parser.getResourceID();
		String xml = null;
		RestResponse response = new RestResponse();

		// By default, set the Response code to SC_OK.
		response.setResponseCode(IRestAPIResponseCodes.SC_OK);

		switch (resourceType) {
		case virtualimage:
			if (resourceId.equals("*")) {
				xml = _virtualizationService.getVirtualImages();
			}
			break;

		case virtualmachine:
			if (resourceId.equals("*")) {
				xml = _virtualizationService.getVirtualMachines();
			} else if (resourceId != null && resourceId.length() > 0) {
				xml = _virtualizationService.getVirtualMachine(resourceId);
			}
			break;

		case vicollection:
			if (resourceId.equals("*")) {
				xml = _virtualizationService.getVirtualImageCollections();
			} else if (resourceId != null && resourceId.length() > 0) {
				xml = _virtualizationService
						.getVirtualImageCollection(resourceId);
			} else if (resourceId == null || "".equals(resourceId)) {
				xml = _virtualizationService.getVirtualImageCollectionIds();
			}
			break;

		case vmcollection:
			if (resourceId.equals("*")) {
				xml = _virtualizationService.getVirtualMachineCollections();
			} else if (resourceId != null && resourceId.length() > 0) {
				xml = _virtualizationService
						.getVirtualMachineCollection(resourceId);
			} else if (resourceId == null || "".equals(resourceId)) {
				xml = _virtualizationService.getVirtualMachineCollectionIds();
			}
		}
		response.setXMLResponse(xml);
		return response;
	}

	public void init(String initProperties, ITracer tracer) throws Exception {
		VirtualizationService.initVI();
		VirtualizationService.initVIC();
		VirtualizationService.initVM();
		VirtualizationService.initVMC();
	}

	public IRestResponse post(URI uri, String XMLBody) {
		// Get the meta data from the URI
		SPIURI parser = SPIURI.vmParse(uri);
		String providerHostName = parser.getProviderHostname();
		int providerPort = parser.getProviderPort();
		URIResourceType resourceType = parser.getResourceTypes()[0];
		String resourceId = parser.getResourceID();
		String xml = null;
		RestResponse response = new RestResponse();

		// By default, set the Response code to SC_OK.
		response.setResponseCode(IRestAPIResponseCodes.SC_OK);
		switch (resourceType) {
		case vmmanager:
			URIResourceType vmMgrResourceType = parser.getResourceTypes()[1];
			switch (vmMgrResourceType) {
			case deploy:
				URIResourceType deployResourceType = parser.getResourceTypes()[2];
				switch (deployResourceType) {
				case vmimage:
					if (resourceId != null && resourceId.length() > 0) {
						/*
						 * Need to handle possible exceptions. For example,
						 * connection of provider error, authentification error,
						 * provider internal error etc., see
						 * com.ibm.rational.test.lm.spi.common.IRestAPIResponseCodes
						 * to get more details. If these errors occur, adaptor
						 * implementation could set plain text as error message
						 * in the response especially for
						 * SC_INTERNAL_SERVER_ERROR.
						 * 
						 * What about localization of the error message from
						 * provider adaptor?
						 * 
						 * <Comments from John Nason in WI 6871>
						 * 
						 * A way is passing client locale information to the
						 * adaptor implementation from RTLM, so they can do
						 * their own localization but that is just a bit
						 * overkill for now.
						 */
						try {
							xml = _virtualizationService.deployImage(resourceId);
							if (xml == null) {
								response.setResponseCode(IRestAPIResponseCodes.SC_NO_CONTENT);
								xml = "The deploying operation is success but no id of the created virtual machine";
							}
						} catch (Exception e) {
							return handleResponseException(response, e);
						}
					} else {
						response.setResponseCode(IRestAPIResponseCodes.SC_BAD_REQUEST);
						xml = "The request of deploying virtual image is not valid";
					}
					break;
				case vicollection:
					if (resourceId != null && resourceId.length() > 0) {
						xml = _virtualizationService
								.deployVirtualImageCollection(resourceId);
						// TODO Handle exception
					}
					break;
				}
				break;
			case capture:
				// TODO Implement
				response.setResponseCode(IRestAPIResponseCodes.SC_NOT_IMPLEMENTED);
				xml = "The service is not implemented";
				break;
			}
			break;
		case virtualmachine:

			break;
		}
		response.setXMLResponse(xml);
		return response;
	}

	public class RestResponse implements IRestResponse {
		private int responseCode = IRestAPIResponseCodes.SC_INTERNAL_SERVER_ERROR;
		private String xmlResponse = "";

		public int getResponseCode() {
			return responseCode;
		}

		public void setResponseCode(int code) {
			responseCode = code;
		}

		public String getXMLResponse() {
			return xmlResponse;
		}

		public void setXMLResponse(String response) {
			xmlResponse = response;
		}
	}
	
	/*
	 * Set response code of the response according to the type of passing
	 * exception and set the XML content of the response to the message of the
	 * exception
	 * 
	 */
	private RestResponse handleResponseException(RestResponse response, Exception ex){
		if (ex instanceof SPIClientConnectionException) {
			response.setResponseCode(IRestAPIResponseCodes.SC_GATEWAY_TIMEOUT);
		} else if (ex instanceof SPIAuthenticationException) {
			response.setResponseCode(IRestAPIResponseCodes.SC_FORBIDDEN);
		} else if (ex instanceof SPIResourceNotFoundException) {
			response.setResponseCode(IRestAPIResponseCodes.SC_NOT_FOUND);
		} else {
			response.setResponseCode(IRestAPIResponseCodes.SC_INTERNAL_SERVER_ERROR);
		}
		response.setXMLResponse(ex.getMessage());
		return response;
	}
}
